/* TestAbortingActivities.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestAbortingActivities.java#6 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.HashMap;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTargetRecs;
import com.ibm.rbapi.RBCustomTemplate;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;

/*
 * This example program will launch a custom script activity on all hosts available 
 * then will cancel the activities 
 * - shows a couple different ways of cancelling - by activity and by activity target.
 * 
 */
public class TestAbortingActivities {
    public static void main(String[] args) 
    throws UnknownHostException, InterruptedException, IOException, RBServerException,
    RBObjectNotFoundException
    {
        
        if (args.length < 3) {
            System.out.println("usage: TestAbortingActivities <ip> <port> <password>");
            return;
        }

        System.out.println("TestAbortingActivities starting.");
        System.out.println(
                "Warning, this test will launch activities on all hosts available.  It will then test aborting the activites.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        
        /* Test RBCustomTemplate using test script below */
        String script =     "Printf(\"Testing testing...\\n\");\n" 
                          + "int progress = 0;\n"
                          + "Printf(\"Activity Parameter a is: %s\\n\", GetActivityParameter(\"a\"));\n"
                          + "for (int i = 0; i < 5; i++) {\n"
                          + "    Printf(\"Sleeping for 5 seconds...\\n\");\n"
                          + "    delay(500);\n"
                          + "    progress = progress + 20;\n"
                          + "    Printf(\"Updating progress to %i\\n\", progress);\n"
                          + "    ReportActivityProgress(progress);\n"
                          + "}\n"
                          + "int ds = SystemInfo.Disks[0]; // size in kilobytes\n"
                          + "SetActivityParameter(\"DiskSize\",(str)ds);\n"
                          + "Printf(\"Disk size is: %i \\n\", ds);\n";
        HashMap pMap = new HashMap();
        pMap.put("a", "param a value");
        pMap.put("b", "param b value");
        RBCustomTemplate rbcTempl = rbServer.registerCustomTemplate("Test Description for custom template", script, pMap);
        System.out.println("Newly added Activity Template of type: Custom Template, description: " + rbcTempl.getDescription());
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = null;
        while (true) {
            System.out.println("\nRetrieving list of available hosts...");
            // Now get list of hosts we can run it on
            hostRecs = rbServer.getHosts("", "");
            
            if (hostRecs.size() == 0) {
                System.out.println("\n\nThere are no hosts found!  This test script needs at least one host to run properly.");
                System.out.println("Please start a host so it boots off the TPMfOSd server and gets registered in the database." +
                        "\nPress enter to continue.");
                inReader.readLine();
            } else {
                break;
            }
        }
        
        /* Test1 - aborting by activity */
        System.out.println("\n#################### Test 1 ############################");
        System.out.println("\nRunning activity with description: TestAbortActivity on all available hosts...");
        RBHost[] targetHosts = hostRecs.subList(0, hostRecs.size());
        Date start = new Date();
        Date stop = new Date(start.getTime()+1000*3600);
        Date expire = new Date(start.getTime()+1000*3600*24);
        RBActivity act = rbServer.registerActivity("TestAbortActivity - running on all hosts.",
                rbcTempl, false, false, pMap, targetHosts, start, stop, expire); 
        
        RBActivityTargetRecs tRecs = act.getActivityTargets();
        System.out.println("### BEFORE the abort, printing out ActivityTarget info:");
        printAllTargetsStatusAndControl(tRecs);
        
        /*
         * Let's abort the activity we just started.
         * Aborting by activity automatically aborts all activity targets associated
         * with that activity.
         */ 
        System.out.println("\nNow we will abort the activity...");
        RBActivity[] aList = new RBActivity[1];
        aList[0] = act;
        boolean[] test1resp = rbServer.abort(aList);    // Abort the activity
        System.out.println("Abort of activity returns: " + test1resp[0]);
        
        System.out.println("\nAbort is done, let's check on the status of those ActivityTargets...");
        tRecs = act.getActivityTargets();       // Reload ActivityTargetRecs info from the server
        System.out.println("### AFTER Abort, printing out ActivityTarget info:");
        printAllTargetsStatusAndControl(tRecs); 
        
        
        /* Test2 - aborting by activity target */
        System.out.println("\n\n#################### Test 2 ############################");
        System.out.println("\nNow let's try aborting by ActivityTarget - We will run 2 activities, and cancel" +
                " one ActivityTarget from each.");
        System.out.println("Running activity with description TestAbortActivity2 on all available hosts...");
        RBActivity act2 = rbServer.registerActivity("TestAbortActivity2 - running on all hosts",
                rbcTempl, false, false, pMap, targetHosts, start, stop, expire);
        System.out.println("Running activity with description TestAbortActivity3 on first available host...");
        RBActivity act3 = rbServer.registerActivity("TestAbortActivity3 - running on one host",
                rbcTempl, false, false, pMap, hostRecs.subList(0,1),    // Run it on the first host in the hostRecs
                start, stop, expire);
        
        System.out.println("\n### BEFORE Abort -- printing out ActivityTarget info for TestAbortActivity2:");
        RBActivityTargetRecs tRecs2 = act2.getActivityTargets();
        printAllTargetsStatusAndControl(tRecs2);
        System.out.println("\n#### BEFORE Abort -- printing out ActivityTarget info for TestAbortActivity3:");
        RBActivityTargetRecs tRecs3 = act3.getActivityTargets();
        printAllTargetsStatusAndControl(tRecs3);
        
        /* We are only going to abort the 1st activity target from each activity */
        System.out.println("Now aborting 1st ActivityTarget from TestAbortActivity2 and the 1st target\n" +
                " from testAbortActivity3.");
        RBActivityTarget[] test2targets = new RBActivityTarget[2];
        test2targets[0] = tRecs2.elementAt(0);
        test2targets[1] = tRecs3.elementAt(0);
        
        boolean[] test2resp = rbServer.abort(test2targets);     // Abort the activity targets.
        System.out.println("\nAbort done.");
        System.out.println("1st ActivityTarget abort returns: " + test2resp[0]);
        System.out.println("2nd ActivityTarget abort returns: " + test2resp[1]);
        
        System.out.println("\n### AFTER Abort -- printing out ActivityTarget info for TestAbortActivity2:");
        tRecs2 = act2.getActivityTargets();
        printAllTargetsStatusAndControl(tRecs2);
        System.out.println("\n#### AFTER Abort -- printing out ActivityTarget info for TestAbortActivity3:");
        tRecs3 = act3.getActivityTargets();
        printAllTargetsStatusAndControl(tRecs3);
        
        System.out.println("\nNow that we're done, remove the template...");
        rbServer.removeTemplate(rbcTempl, true);
        System.out.println("\nDone.");
    }
    
    private static void printAllTargetsStatusAndControl(RBActivityTargetRecs tRecs) 
    throws RBServerException, RBObjectNotFoundException 
    {
        RBActivityTarget[] targets = tRecs.subList(0, tRecs.size());
        for (int i=0; i<tRecs.size(); i++) { 
            RBActivityTarget actTarget = targets[i];
            System.out.println("## ActivityTarget" + i + " control: " + actTarget.getControl());
            System.out.println("## ActivityTarget" + i + " status: " + actTarget.getStatus());
            System.out.println("##");
        }
    }
}
