/* TestAbortingActivitiesForHost.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestAbortingActivitiesForHost.java#1 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityRecs;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTargetRecs;
import com.ibm.rbapi.RBCustomTemplate;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;

/*
 * This example shows aborting all activities for a particular host,
 * and also shows an example of removing activities.
 */
public class TestAbortingActivitiesForHost {

    public static void main(String[] args) 
    throws UnknownHostException, InterruptedException, IOException, RBServerException,
    RBObjectNotFoundException
    {
        
        if (args.length < 3) {
            System.out.println("usage: TestAbortingActivitiesForHost <ip> <port> <password>");
            return;
        }

        System.out.println("TestAbortingActivitiesForHost starting.");
        System.out.println(
                "Warning, this test will launch activities on a host.  It will then test aborting the activites.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        
        /* Make some templates to use */
        /* use RBCustomTemplate using test script below */
        String script =     "Printf(\"Testing testing...\\n\");\n" 
                          + "Printf(\"Sleeping for 25 seconds...\\n\");\n"
                          + "    delay(2500);\n"
                          + "Printf(\"Done\\n\");\n";
        RBCustomTemplate rbcTempl = rbServer.registerCustomTemplate("Test Description for custom template", script, null);
        System.out.println("Newly added Activity Template of type: Custom Template, description: " + rbcTempl.getDescription());
        
        /* Make another template */
        String script2 =    "Printf(\"Another test...\\n\");\n"
                          + "Printf(\"Sleeping for 20 seconds...\\n\");\n"
                          + "    delay(2000);\n";
        RBCustomTemplate rbcTempl2 = rbServer.registerCustomTemplate("Test Description TWO for custom template", script2, null);
        System.out.println("Newly added Activity Template of type: Custom Template, description: " + rbcTempl.getDescription());
        
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = null;
        while (true) {
            System.out.println("\nRetrieving list of available hosts...");
            // Now get list of hosts we can run it on
            hostRecs = rbServer.getHosts("", "");
            
            if (hostRecs.size() == 0) {
                System.out.println("\n\nThere are no hosts found!  This test script needs at least one host to run properly.");
                System.out.println("Please start a host so it boots off the TPMfOSd server and gets registered in the database." +
                "\nPress enter to continue.");
                inReader.readLine();
            } else {
                break;
            }
        }
        
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress() 
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host you wish to run the test script on: \n");
            inReader = new BufferedReader(new InputStreamReader(System.in));
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = availHosts[hostIndex];
        
        System.out.println("\n#################### Test 1 ############################");
        System.out.println("\nRunning 2 activities on the host...");
        RBActivity act = rbServer.registerActivity("TestAbortActivityForHost1.",
                rbcTempl, false, false, null, rbhost);
        
        RBActivity act2 = rbServer.registerActivity("TestAbortActivityForHost2", 
                rbcTempl2, false, false, null, rbhost);
        
        RBActivityTargetRecs tRecs = act.getActivityTargets();
        System.out.println("### BEFORE the abort, printing out #1 ActivityTarget info:");
        printAllTargetsStatusAndControl(tRecs);
        
        RBActivityTargetRecs tRecs2 = act2.getActivityTargets();
        System.out.println("### BEFORE the abort, printing out #2 ActivityTarget info:");
        printAllTargetsStatusAndControl(tRecs2);
        
        // Now let's abort all activitys for this host
        System.out.println("\nNow we will abort all Activities for our host"); 
        rbhost.abortAllActivities();
        
        
        System.out.println("\nAbort is done, let's check on the status of those ActivityTargets...");
        tRecs = act.getActivityTargets();       // Reload ActivityTargetRecs info from the server
        System.out.println("### AFTER Abort, printing out #1 ActivityTarget info:");
        printAllTargetsStatusAndControl(tRecs); 
        tRecs2 = act2.getActivityTargets();       // Reload ActivityTargetRecs info from the server
        System.out.println("### AFTER Abort, printing out #2 ActivityTarget info:");
        printAllTargetsStatusAndControl(tRecs2); 
        
        // Test the remove method
        act.remove();
        act2.remove();
        RBActivityRecs ar = rbhost.getAllActivities();
        if (ar.size() != 0) {
            System.out.println("\n!!!!!ERROR!!!!! activities not removed");
        } else {
            System.out.println("\nActivities removed as expected");
        }
        
        System.out.println("\nNow that we're done, remove the templates...");
        rbServer.removeTemplate(rbcTempl, true);
        rbServer.removeTemplate(rbcTempl2, true);
        System.out.println("\nDone.");
    }
    
    private static void printAllTargetsStatusAndControl(RBActivityTargetRecs tRecs) 
    throws RBServerException, RBObjectNotFoundException 
    {
        RBActivityTarget[] targets = tRecs.subList(0, tRecs.size());
        for (int i=0; i<tRecs.size(); i++) { 
            RBActivityTarget actTarget = targets[i];
            System.out.println("## ActivityTarget control: " + actTarget.getControl());
            System.out.println("## ActivityTarget status: " + actTarget.getStatus());
            System.out.println("##");
        }
    }
    
}
