/* TestAgentScript.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestAgentScript.java#3 $
 */
package examples;

import java.net.InetAddress;
import java.util.Date;
import java.util.HashMap;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBCustomTemplate;
import com.ibm.rbapi.RBEvent;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBServer;

public class TestAgentScript {

    /*
     * Run a RemboC script on a TPMFOSD server, using rbagent
     */
    public static void main(String[] args) 
    {
        
        if (args.length < 3) {
            System.out.println("usage: TestAgentScript <ip> <port> <password>");
            return;
        }
        
        try {
            System.out.println("TestAgentScript starting.");
            InetAddress ip = InetAddress.getByName(args[0]);
            int port = Integer.parseInt(args[1]);
            String secret = args[2];
            RBServer rbServer = new RBServer(ip, port, secret); 
            // Abort if the server is not ready after 15 seconds
            rbServer.isReady(15000);
            
            /* Script for rbagent
             * see http://www.rembo.com/rbo/docs/RemboCIndex.html for Rembo-C function reference 
             * 
             * NOTE:  This script is intended for an rbagent running on a Windows machine
             */
            String script =     "ActivityLog(3,\"TestAgentScript starting...\");\n"
                + "str destroot = \"local://root/c$/temp\";\n"                      // Rembo-C string variable definition
                + "if (FileExists(destroot)) {\n"                                   // FileExists: Rembo-C function to check file existence
                + "    ActivityLog(3,Strf(\"%s already exists\",destroot));\n"      // Strf: string formatting
                + "} else {\n"
                + "    MkDir(destroot);\n"                                          // MkDir: directory creation
                + "    ActivityLog(3,Strf(\"%s has been created\",destroot));\n"
                + "}\n"
                + "str destfile = StrChDir(destroot,\"agent.exe\");\n"              // StrChDir: path concatenation
                + "if (FileExists(destfile)) {\n"
                + "    RemoveFile(destfile);\n"                                     // RemoveFile: file deletion
                + "    ActivityLog(3,Strf(\"%s has been deleted\",destfile));\n"
                + "}\n"
                + "str progdir = GetActivityParameter(\"installdir\");\n"           // GetActivityParameter: special for custom templates
                + "ActivityLog(3,Strf(\"installdir is %s\",progdir));\n"
                + "str agentfile = StrChDir(progdir,\"rbagent.exe\");\n"
                + "if (FileExists(agentfile)) {\n"
                + "    CopyFile(agentfile,destfile);\n"                             // CopyFile: copies whole files
                + "    ActivityLog(3,Strf(\"rbagent.exe copied to %s...\",destfile));\n"
                + "} else {\n"
                + "    ActivityLog(3,Strf(\"Cannot find: %s\",agentfile));\n"
                + "}\n"
                + "str destdir = \"local://root/c$/temp/rad\";\n"
                + "if (FileExists(destdir)) {\n"
                + "    RemoveTree(destdir);\n"                                      // RemoveTree: deletes whole directories
                + "    ActivityLog(3,Strf(\"%s has been deleted\",destdir));\n"
                + "}\n"
                + "CopyTree(\"net://global/rad\",destdir);\n"                       // CopyTree: copy whole directories
                + "ActivityLog(3,Strf(\"directory copied to %s\",destdir));\n"
                + "ActivityLog(3,\"TestAgentScript successful.\");\n"
                + "ActivityTerminate(2); // (0:fail, 1:cancel, 2:success)\n";       // ActivityTerminate: special for custom templates
            HashMap pMap = new HashMap();
            
            // Illustrates parameter passing, you may want to change it on non-french Windows!
            pMap.put("installdir", "local://root/c$/program files/fichiers communs/ibm tivoli/");
            RBCustomTemplate rbcTempl = rbServer.registerCustomTemplate(
                    "TestAgentScript templ", script, pMap, RBCustomTemplate.AGENT_ENVIRONMENT);
            System.out.println("Custom Template created, description: " + rbcTempl.getDescription());
            System.out.println("Script:");
            System.out.println(script);

            RBHostRecs hostRecs = rbServer.getHosts("[IP]='"+args[0]+"'", "");
            if (hostRecs.size() == 0) {
                System.out.println("\nCannot find RBHost "+args[0]+" please make sure that rbagent is running.");
            } else {
                RBHost rbhost = hostRecs.elementAt(0);
                System.out.println("\n\nNow we will schedule the activity "
                        + "on the host with IP: " + rbhost.getIPAddress()
                        + " and description" + rbhost.getDescription());
                RBActivity newAct = rbServer.registerActivity("TestAgentScript act", 
                        rbcTempl, false, false, null, rbhost);
                
                System.out.println("Activity scheduled, waiting for completion...\n");
                Date lastTime = new Date();
                boolean finished = false;
                while (!finished) {
                    RBEvent newevents[] = rbServer.getEvents(lastTime);
                    for (int i=0; i<newevents.length && !finished; i++) {
                        RBEvent cur = newevents[i];
                        lastTime = cur.getUpdateTime(); // Update last events found time.
                        if (cur.belongsTo(newAct)) {
                            if (!cur.getStatus().equals("PENDING") && !cur.getStatus().equals("INPROGRESS")) {
                                System.out.println("Activity finished with status:"+cur.getStatus());
                                finished = true;
                            }
                        }
                    }
                    if (!finished) {
                        try { Thread.sleep(5000); } catch (InterruptedException e) {} // try again after 5 seconds
                    }
                }
            }
            // Now remove the template
            System.out.println("\nRemoving the previously created template...");
            rbServer.removeTemplate(rbcTempl, true);
            System.out.println("Done.");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
