/* TestOpenActivity.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestOpenActivity.java#5 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBCustomTemplate;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;

/*
 * This test class schedules an "open" activity.
 * 
 * The open activity will run once and only once on all hosts after they
 * PXE boot from the server.  New hosts will automatically run the open activity 
 * as well.
 * 
 * If you delete a host and then add it again, it is considered as a new host,
 * so it will then run the open activity when you PXE boot it.
 */
public class TestOpenActivity {

    public static void main(String[] args) throws UnknownHostException, 
    InterruptedException, IOException, RBServerException {
        
        if (args.length < 3) {
            System.out.println("usage: TestOpenActivity <ip> <port> <password>");
            return;
        }
        
        System.out.println("TestOpenActivity starting.  This test class will" +
                " run a test rembo-c script on all hosts that PXE boot from the" +
                " TPMfOSd server once.\n");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
            
        /* We will test an open activity with an RBCustomTemplate */
        String script = "Printf(\"TESTING OPEN ACTIVITY.\\n\");\n" 
                + "int ds = SystemInfo.Disks[0]; // size in kilobytes\n"
                + "SetActivityParameter(\"DiskSize\",(str)ds);\n"
                + "Printf(\"Disk size is: %i \\n\", ds);\n";
        RBCustomTemplate rbcTempl = rbServer.registerCustomTemplate(
                "Test custom template for OpenAct test", script, null);
        System.out.println("Newly added Activity Template of type: Custom Template, " +
                "description: " + rbcTempl.getDescription());
            
        System.out.println("\n\nNow we will schedule an Open Activity using" +
                " the previously created CustomTemplate.");
            
        /* Schedule the open Activity */
        Date now = new Date();
        Date tomorrow = new Date(now.getTime() + (24*3600*1000));
        RBActivity openAct = rbServer.registerOpenActivity("testOpenActivity", 
                rbcTempl, true, null, now, tomorrow, tomorrow);
        System.out.println("\nCreated new Open activity to run immediately.");
        printActivity(openAct); 
        
            
        System.out.println("This test class will now pause so you can PXE " 
                + "boot hosts that you want to see the script run on.\n" 
                + "Press enter and the class will clean up and exit.\n"
                + "     (The new RBCustomTemplate will be removed, also removing\n"
                + "     the new open activity and all its activity targets from\n" 
                + "     the system.)\n\n");
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        inReader.readLine();
               
        System.out.println("\n Removing CustomTemplate, description: " + 
                rbcTempl.getDescription() + "...");
        rbServer.removeTemplate(rbcTempl, true);
        System.out.print("\n TestOpenActivity done.");
    }
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
    
    public static void printActivity(RBActivity activity) {
        System.out.println("\n### RBActivity - printing out ###");
        System.out.println("###  getDescription(): " + activity.getDescription());
        System.out.println("###  getDoneDate(): " + activity.getDoneDate());
        System.out.println("###  getExecDate(): " + activity.getExecDate());
        System.out.println("###  getExpireDate(): " + activity.getExpireDate());
        System.out.println("###  getProgress(): " + activity.getProgress());
        System.out.println("###  isSuspendOnError(): " + activity.isSuspendOnError());
        System.out.println("###  isOpenActivity(): " + activity.isOpenActivity());
    }
}
