/* TestRBCustomTemplateUsingVariant.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestRBCustomTemplateUsingVariant.java#1 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTemplate;
import com.ibm.rbapi.RBCustomTemplate;
import com.ibm.rbapi.RBCustomVariant;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.exception.RBTimeoutException;

/*
 * This example is similar to TestRBCustomTemplate, but uses an RBCustomVariant
 * to override some settings from the RBCustomTemplate.
 */
public class TestRBCustomTemplateUsingVariant {

    public static void main(String[] args) throws UnknownHostException, InterruptedException, IOException, RBTimeoutException {
        
        if (args.length < 3) {
            System.out.println("usage: TestRBCustomTemplate <ip> <port> <password>");
            return;
        }
        
        try {

            System.out.println("TestRBCustomTemplate starting.");
            InetAddress ip = InetAddress.getByName(args[0]);
            int port = Integer.parseInt(args[1]);
            String secret = args[2];
            RBServer rbServer = new RBServer(ip, port, secret); 
            // Abort if the server is not ready after 15 seconds
            rbServer.isReady(15000);
            
            /* Test RBCustomTemplate using test script below 
             * 
             *  **We are intentionally putting a script that will fail here 
             *  - the idea is to override the script in this example when
             *  we schedule the activity.  We will override with a good script.**
             */
            
            String script = "Printf(\"BAD SCRIPT!!\");\n"
                          + "ActivityTerminate(0);\n";
            HashMap pMap = new HashMap();
            pMap.put("a", "param a value");
            pMap.put("b", "param b value");
            RBCustomTemplate rbcTempl = rbServer.registerCustomTemplate(
                    "Description for custom template", script, pMap, 
                    RBCustomTemplate.KERNEL_ENVIRONMENT);
            System.out.println("Newly added Activity Template of type: Custom Template, description: " + rbcTempl.getDescription());
            
            
            /* Set an end action of HDBOOT for the template */
            rbcTempl.setEndAction(RBActivityTemplate.HDBOOT); 
            rbcTempl.update();
            
            BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
            RBHostRecs hostRecs = null;
            while (true) {
                System.out.println("\nRetrieving list of available hosts...");
                // Now get list of hosts we can run it on
                hostRecs = rbServer.getHosts("", "");
                
                if (hostRecs.size() == 0) {
                    System.out.println("\n\nThere are no hosts found!  This test script needs at least one host to run properly.");
                    System.out.println("Please start a host so it boots off the TPMfOSd server and gets registered in the database." +
                    "\nPress enter to continue.");
                    inReader.readLine();
                } else {
                    break;
                }
            }
            
            RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
            for (int i=0; i<availHosts.length; i++) {
                System.out.println(i + ") " + availHosts[i].getIPAddress()
                        + " MAC: " + availHosts[i].getMacAddress() 
                        + " Description: " + availHosts[i].getDescription());
            }
            
            int hostIndex = -1;
            do {
                System.out.println("Please enter the number for the host you wish to run the test script on: \n");
                inReader = new BufferedReader(new InputStreamReader(System.in));
                hostIndex = Integer.parseInt(inReader.readLine().trim());
            } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
            
            RBHost rbhost = hostRecs.elementAt(hostIndex);
            
            /*
             * Here is an example of using a variant to override settings from the
             * template at runtime.  The template itself remains unchanged.
             */
            RBCustomVariant rbcVariant = rbcTempl.getVariant();
            System.out.println("\nEarlier we set our RBCustomTemplate to have an end action\n" +
                    "of HDBOOT, meaning after the activity is done, if there are no other\n" +
                    "pending activities for the host, it will boot from the hard drive.\n" +
                    "We can now override this template setting if you wish - Enter a number\n" +
                    "to choose one of the following: \n" +
                    "1) Set end action to POWEROFF\n" +
                    "2) Set end action to REBOOT\n" +
                    "3) Set end action to STAY\n" + 
                    "4) Continue, use template setting of HDBOOT\n\n");
            String endActChoice = inReader.readLine().trim();
            if (endActChoice.equals("1")) {
                System.out.println("Setting end action to POWEROFF...");
                rbcVariant.setEndAction(RBActivityTemplate.POWEROFF);
            } else if (endActChoice.equals("2")) {
                System.out.println("Setting end action to REBOOT...");
                rbcVariant.setEndAction(RBActivityTemplate.REBOOT);
            } else if (endActChoice.equals("3")) {
                System.out.println("Setting end action to STAY...");
                rbcVariant.setEndAction(RBActivityTemplate.STAY);
            } else {
                System.out.println("Leaving end action as HDBOOT...");
            }
            
            String newScript =     "Printf(\"NEW TEST SCRIPT...\\n\");\n" 
                + "int progress = 0;\n"
                + "Printf(\"Activity Parameter a is: %s\\n\", GetActivityParameter(\"a\"));\n"
                + "for (int i = 0; i < 10; i++) {\n"
                + "    Printf(\"Sleeping for 3 seconds...\\n\");\n"
                + "    delay(300);\n"
                + "    progress = progress + 10;\n"
                + "    Printf(\"Updating progress to %i\\n\", progress);\n"
                + "    ReportActivityProgress(progress);\n"
                + "}\n"
                + "int ds = SystemInfo.Disks[0]; // size in kilobytes\n"
                + "SetActivityParameter(\"DiskSize\",(str)ds);\n"
                + "Printf(\"Disk size is: %i \\n\", ds);\n";
            rbcVariant.setScript(newScript);
            
            System.out.println("\n\nNow we will schedule the previously created " +
                    "ActivityTemplate to run on the host with IP: " 
                    + rbhost.getIPAddress()
                    + " and description" + rbhost.getDescription());
            
            pMap.put("a", "NEW VALUE");    
            
            /* 
             * Schedule an Activity on user specified host - using the variant
             * in case the user wants to override the end action from the template.
             * Note that you can pass in the original template if you want to 
             * leave the original template settings alone
             */
            RBActivity newAct = rbServer.registerActivity("TestActivity using RBCustomTemplate", 
                    rbcVariant, false, true, pMap, rbhost);
            System.out.println("\nCreated new activity to run immediately on host " +
                    "with IP: " + rbhost.getIPAddress());
            printActivity(newAct); 
            
            
            System.out.println("\nPress enter when the activity has finished running\n" +
                    "on the target, and then this program will then print out some info,\n" +
                    "remove the template and exit.\n\n");
            inReader.readLine();
            
            // Let's just print out some info about the target if it's done first...
            // There should only be one target as we scheduled only one.
            RBActivityTarget target = newAct.getActivityTargets().elementAt(0);
            System.out.println("ActivityTarget status: " + target.getStatus());
            System.out.println("ActivityTarget last progress msg: " + target.getLastProgressMsg());
            Map returnedParams = target.getParams();
            if (returnedParams != null) {
                String diskSize = (String) returnedParams.get("DiskSize");
                if (diskSize != null) {
                    System.out.println("Disk size returned from custom script is: " + diskSize);
                } else {
                    System.out.println("Error, did not get disk size returned from custom script.");
                }
            } else {
                System.out.println("ActivityTarget returned no params.");
            }
            
            // Now remove the template
            System.out.println("\nRemoving the previously created template...");
            rbServer.removeTemplate(rbcTempl, true);
            System.out.println("Done.");
        } catch (RBServerException rbEx) {
            System.out.println("Error with server " );
            rbEx.printStackTrace();
        } catch (RBObjectNotFoundException nfEx) {
            nfEx.printStackTrace();
        }
        System.out.print("\n TestRBCustomTemplate done.");
    }
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
    
    public static void printActivity(RBActivity activity) {
        System.out.println("\n### RBActivity - printing out ###");
        System.out.println("###  getDescription(): " + activity.getDescription());
        System.out.println("###  getDoneDate(): " + activity.getDoneDate());
        System.out.println("###  getExecDate(): " + activity.getExecDate());
        System.out.println("###  getExpireDate(): " + activity.getExpireDate());
        System.out.println("###  getProgress(): " + activity.getProgress());
        System.out.println("###  isSuspendOnError(): " + activity.isSuspendOnError());
    }
    
    public static void printActivityTarget(RBActivityTarget target) {
        System.out.println("\n### RBActivityTarget - printing out ###");
        System.out.println("###  getProgress(): " + target.getProgress());
        System.out.println("###  getStatus(): " + target.getStatus());
//        System.out.println("###  getStatusDate(): " + target.getStatusDate());
//        System.out.println("###  getControl(): " + target.getControl());
    }

}
