/* TestRBDeviceBlankingTemplate.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestRBDeviceBlankingTemplate.java#2 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;

import com.ibm.rbapi.RBDeviceBlankingTemplate;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestRBDeviceBlankingTemplate {

    public static void main(String[] args) 
    throws UnknownHostException, RBServerException, RBObjectNotFoundException, IOException, RBTimeoutException
    {
        if (args.length < 3) {
            System.out.println("usage: TestRBDeviceBlankingTemplate <ip> <port> <password>");
            return;
        }
        System.out.println("TestRBDeviceBlankingTemplate starting...");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String token = args[2];
        RBServer rbServer = new RBServer(ip, port, token); 
        System.out.println("Created rbServer.");
        System.out.println("This test will attempt to blank the 1st disk on a host of " +
                "your choice doing one pass with the pattern 123456");
        // Abort if the server is not ready after 15 seconds
        rbServer.isReady(15000);
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = null;
        while (true) {
            System.out.println("\nRetrieving list of available hosts...");
            // Now get list of hosts we can run it on
            hostRecs = rbServer.getHosts("", "");
            
            if (hostRecs.size() == 0) {
                System.out.println("\n\nThere are no hosts found!  This test script needs at least one host to run properly.");
                System.out.println("Please start a host so it boots off the TPMfOSd server and gets registered in the database." +
                "\nPress enter to continue.");
                inReader.readLine();
            } else {
                break;
            }
        }
        
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress() 
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host you wish to blank the disks of: \n");
            inReader = new BufferedReader(new InputStreamReader(System.in));
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = availHosts[hostIndex];
        
        
        System.out.println("\n WARNING - are you sure you wish to proceed?  This program will\n" +
                "run an activity to blank the first disk on your host entirely!\n\n" +
                "Enter y to continue or any other key to exit:\n");
        String answer = inReader.readLine().trim().toLowerCase();
       
        if (!answer.equals("y"))  {
            System.out.println("\nAborting program, will NOT run blanking activity.");
            System.exit(1);
        }
        
        System.out.println("\nRegistering Device blanking template...");
        // Create pattern of one int - so we will be performing a blanking activity
        // with only one pass.
        int[] ourPattern = new int[] {123456}; 
        RBDeviceBlankingTemplate devBlankTempl = 
            rbServer.registerDeviceBlankingTemplate("testDevBlank desc", ourPattern);
        
        System.out.println("\nRegistering an activity to immediately perform the " +
                "device blanking on our host: " 
                + rbhost.getIPAddress()
                + " MAC: " + rbhost.getMacAddress() 
                + " Description: " + rbhost.getDescription() + " ...");
        
        /* 
         * Make sure to pass in a param map with "disk" specified - in this
         * case we are using disk 0
         */
        HashMap params = new HashMap();
        params.put("disk", "0");
        rbServer.registerActivity("testDevBlank activity", devBlankTempl, 
                false, false, params, rbhost);
        
        System.out.println("Activity is registered.  Press enter when the activity " +
                "has completed and this will remove the template and exit.");
        inReader.readLine();
        
        System.out.println("\nRemoving device blanking template...");
        rbServer.removeTemplate(devBlankTempl, true);
        System.out.println("Done.");
    }
}
