/* TestRBHosts.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2006
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestRBHosts.java#11 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;

import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestRBHosts {

    public static void main(String[] args) throws UnknownHostException, RBTimeoutException {
        
        if (args.length < 3) {
            System.out.println("usage: TestRBHosts <ip> <port> <password>");
            return;
        }
        
        try {
            System.out.println("TestRBHosts starting.");
            InetAddress ip = InetAddress.getByName(args[0]);
            int port = Integer.parseInt(args[1]);
            String token = args[2];
            
            RBServer rbServer = new RBServer(ip, port, token); 
            System.out.println("Created rbServer.\n");
            // Abort if the server is not ready after 15 seconds
            rbServer.isReady(15000);
            
            System.out.println("Calling RB Host methods...\n");
            
            // Register new host
            System.out.println("\n calling registerHost(10.10.1.100, , , )");
            RBHost rbHost = rbServer.registerHost("10.10.1.100", "", "", "");
            // Display list of all hosts
            System.out.println("\n calling getHosts( , )");
            RBHostRecs rbHostRecs = rbServer.getHosts("", "");
            
            System.out.println("getHosts returned " + rbHostRecs.size() + " hosts.\n");
            
            // Update the host
            rbHost.setDescription("NEW DESCRIPTION");
            
            rbHost.update();
            System.out.println(" calling update() on the RBHost after setting description to NEW DESCRIPTION");
            System.out.println("Now rbhost is: ");            
            printOutRBHost(rbHost);        
            
            // remove the host we previously created
            System.out.println("\n calling remove() on the RBHost with the host created before - 10.10.1.100");
            rbHost.remove();
            // Display list of all hosts again
            System.out.println("\n calling getHosts( , ) again.");
            rbHostRecs = rbServer.getHosts("", "");
            
            if (rbHostRecs.size() >= 3) {
                System.out.println("\n calling rbhostRecs.elementAt(1)");
                RBHost rbHost2 = rbHostRecs.elementAt(1);      
                System.out.println("rbhost at 1 is: ");
                printOutRBHost(rbHost2);
                
                System.out.println("\n calling rbhostRecs.subList(0, 3)");
                RBHost[] rbHosts = rbHostRecs.subList(0, 3);
                for (int i=0; i< rbHosts.length; i++) {
                    System.out.println(" rbHosts[" + i + "] is: ");
                    printOutRBHost(rbHosts[i]);
                }
                
                System.out.println("\n\nPrinting out host + " + 2 + " in full: ");
                printOutRBHostInFull(rbHosts[1]);
            }
            
            // Offer choice to remove a host
            try {
                RBHost[] rbHosts = rbHostRecs.subList(0, rbHostRecs.size());
                for (int i=0; i< rbHosts.length; i++) {
                    RBHost current = rbHosts[i];
                    System.out.println(" rbHosts[" + i + "] is: "
                            +current.getIPAddress()+" ["+current.getMacAddress()+"]");
                }
                BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
                int choice = -1;
                do {
                    System.out.println("Please enter the number for the host you wish to remove (letter to skip).\n");
                    choice = Integer.parseInt(inReader.readLine().trim());
                } while ( choice < 0 || choice >= rbHosts.length );
                rbHosts[choice].remove();
            } catch (NumberFormatException e) {
            } catch (IOException ee) {
            }
        
        } catch (RBServerException rbEx) {
            rbEx.printStackTrace();
        } catch (RBObjectNotFoundException nfEx) {
            nfEx.printStackTrace();
        }
    }
    
    static void printOutRBHost(RBHost rbHost) {
        System.out.println("### RBHost - printing out ###");
        System.out.println("###  getHostName(): " + rbHost.getHostName());
        System.out.println("###  getDescription(): " + rbHost.getDescription());
        System.out.println("###  getDeplCount(): " + rbHost.getDeplCount());
        System.out.println("###  getIPAddress(): " + rbHost.getIPAddress());
        System.out.println("###  getMacAddress(): " + rbHost.getMacAddress());
        System.out.println("###  getModel(): " + rbHost.getModel());
        System.out.println("###  getPlatform(): " + rbHost.getPlatform());
        System.out.println("###  getSerialNumber(): " + rbHost.getSerialNumber());
        System.out.println("###  getUUID(): " + rbHost.getUUID());
        System.out.println("###  getIdentDate(): " + rbHost.getIdentDate());
        System.out.println("###  getStatusDate(): " + rbHost.getStatusDate().getTime() + "(now " + new Date().getTime() + ")");
    }
    
    static void printOutRBHostInFull(RBHost rbHost) {
        System.out.println("### RBHost - printing out ###");
        System.out.println("###  getAdministratorName(): " + rbHost.getAdministratorName());
        System.out.println("###  getAdminPasswd(): " + rbHost.getAdminPasswd());
        System.out.println("###  getBitsPerPel(): " + rbHost.getBitsPerPel());
        System.out.println("###  getDeplCount(): " + rbHost.getDeplCount());
        System.out.println("###  getDeployGroupID(): " + rbHost.getDeployGroupID());
        System.out.println("###  getDescription(): " + rbHost.getDescription());
        System.out.println("###  getDNSDomain(): " + rbHost.getDNSDomain());
        System.out.println("###  getDnsOrder(): " + rbHost.getDnsOrder());
        System.out.println("###  getDnsServer1(): " + rbHost.getDnsServer1());
        System.out.println("###  getDnsServer2(): " + rbHost.getDnsServer2());
        System.out.println("###  getDnsServer3(): " + rbHost.getDnsServer3());
        System.out.println("###  isEnableIPv6(): " + rbHost.isEnableIPv6());
        System.out.println("###  getFullName(): " + rbHost.getFullName());
        System.out.println("###  getGateway(): " + rbHost.getGatewayAddress());
        System.out.println("###  getGroupID(): " + rbHost.getGroupID());
        System.out.println("###  getHostName(): " + rbHost.getHostName());
        System.out.println("###  getIdentDate(): " + rbHost.getIdentDate());
        System.out.println("###  getIdentScope(): " + rbHost.getIdentScope());
        System.out.println("###  getIPAddress(): " + rbHost.getIPAddress());
        System.out.println("###  getIpSettings(): " + rbHost.getIPSettings());    
        System.out.println("###  getJoinDomPass(): " + rbHost.getJoinDomPass());    
        System.out.println("###  getJoinDomUser(): " + rbHost.getJoinDomUser());    
        System.out.println("###  getKAdminServer(): " + rbHost.getKAdminServer());    
        System.out.println("###  getKDCServer1(): " + rbHost.getKDCServer1());    
        System.out.println("###  getKDCServer2(): " + rbHost.getKDCServer2());    
        System.out.println("###  getKDCServer3(): " + rbHost.getKDCServer3());    
        System.out.println("###  getKerberosRealm(): " + rbHost.getKerberosRealm());    
        System.out.println("###  getLdapProfile(): " + rbHost.getLdapProfile());    
        System.out.println("###  getLocale(): " + rbHost.getLocale());    
        System.out.println("###  getMacAddress(): " + rbHost.getMacAddress());
        System.out.println("###  getModel(): " + rbHost.getModel());
        System.out.println("###  getMonitorLayout(): " + rbHost.getMonitorLayout());
        System.out.println("###  getMonitorLocation(): " + rbHost.getMonitorLocation());
        System.out.println("###  getNameServer(): " + rbHost.getNameServer());
        System.out.println("###  getNameService(): " + rbHost.getNameService());
        System.out.println("###  getNetMask(): " + rbHost.getNetmaskAddress());
        System.out.println("###  getOrgName(): " + rbHost.getOrgName());
        System.out.println("###  getOrgUnit(): " + rbHost.getOrgUnit());
        System.out.println("###  getPlatform(): " + rbHost.getPlatform());
        System.out.println("###  getProductKey(): " + rbHost.getProductKey());
        System.out.println("###  getPxeBootMode(): " + rbHost.getPxeBootMode());
        System.out.println("###  getRemboLockout(): " + rbHost.getRemboLockout());
        System.out.println("###  getRemboOptions(): " + rbHost.getRemboOptions());
        System.out.println("###  getRemboServer(): " + rbHost.getRemboServer());
        System.out.println("###  getScopeName(): " + rbHost.getScopeName());
        System.out.println("###  getSerialNumber(): " + rbHost.getSerialNumber());
        System.out.println("###  getSrvHostID(): " + rbHost.getSrvHostID());
        System.out.println("###  getSrvHostID2(): " + rbHost.getSrvHostID2());
        System.out.println("###  getStatus(): " + rbHost.getStatus());    
        System.out.println("###  getStatusDate(): " + rbHost.getStatusDate());    
        System.out.println("###  getSystemID(): " + rbHost.getSystemID());    
        System.out.println("###  getTaskID(): " + rbHost.getTaskID());    
        System.out.println("###  getTermInfo(): " + rbHost.getTermInfo());    
        System.out.println("###  getTimeServer(): " + rbHost.getTimeServer());    
        System.out.println("###  getTimeZone(): " + rbHost.getTimeZone());    
        System.out.println("###  getUserCateg0(): " + rbHost.getUserCateg0());    
        System.out.println("###  getUserCateg1(): " + rbHost.getUserCateg1());    
        System.out.println("###  getUserCateg2(): " + rbHost.getUserCateg2());    
        System.out.println("###  getUserCateg3(): " + rbHost.getUserCateg3());    
        System.out.println("###  getUserDomain(): " + rbHost.getUserDomain());    
        System.out.println("###  getUserID(): " + rbHost.getUserID());    
        System.out.println("###  getUserName(): " + rbHost.getUserName());    
        System.out.println("###  getUUID(): " + rbHost.getUUID());
        System.out.println("###  getVRefresh(): " + rbHost.getVRefresh());
        System.out.println("###  getWinsServer1(): " + rbHost.getWinsServer1());
        System.out.println("###  getWinsServer2(): " + rbHost.getWinsServer2());
        System.out.println("###  getXResolution(): " + rbHost.getXResolution());
        System.out.println("###  getYResolution(): " + rbHost.getYResolution());
    }
}
