/* TestRBPrebootExecuteTemplate.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestRBPrebootExecuteTemplate.java#11 $
 */
package examples;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBFile;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBPrebootExecuteTemplate;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBSoftwarePackage;
import com.ibm.rbapi.RBSoftwarePackageRecs;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestRBPrebootExecuteTemplate {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException , RBTimeoutException
    {
        
        // If the test program is called with 4 arguments consider that args[3] is
        // the floppy image filename in files/global/updates (deprecated).
        // If there are only three arguments, get the list of DOS images from the
        // server and let the user choose one.
        
        if (args.length < 3 || args.length > 4) {
            System.out.println("usage: TestRBPrebootExecuteTemplate <ip> <port> <password> [<floppyimagefile>]\n");
            System.out.println("  where floppyimagefile is the filename of the floppy image on your TPMfOSd server in files/global/updates");
            return;
        }

        System.out.println("TestRBPrebootExecuteTemplate starting. We will test booting "
                +"a DOS floppy image on a host machine.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        // Abort if the server is not ready after 15 seconds
        rbServer.isReady(15000);
        
        int ramDiskSize = 2048;
        
        /* Uncomment these lines if you want to copy over edit.com from c:/windows/system32/ */
//        RBFile[] files2Add = new RBFile[3];
//        files2Add[2] = new RBFile("edit.com", readFile("C:/windows/system32/edit.com"));
        RBFile[] files2Add = new RBFile[2];
        files2Add[0] = new RBFile("TEMP/file0.txt", "This is file 0 contents. Netinfo is: {$NetInfo.IPAddress$}.");
//        files2Add[1] = new RBFile("autoexec.bat", readFile("C:/TEMP/autoexec.bat"));
        
        /* We are going to override the autoexec.bat on the floppy image with this one */
        files2Add[1] = new RBFile("autoexec.bat", 
                "@echo off \r\n"
                + "set CLIENT_IP={$NetInfo.IPAddress$} \r\n"
                + "set SERVER_IP={$HostInfo.RemboServer$} \r\n"
                + "set GATEWAY_IP={$NetInfo.DefaultGateway$} \r\n"
                + "set SUBNETMASK={$NetInfo.SubnetMask$} \r\n"
                + "set BOOTMAC={$StrReplace(NetInfo.MACAddress, \" \", \"\")$} \r\n"
                + "set UUID={$DMISystemInfo().uuid$} \r\n"
                + "echo CLIENT_IP is: %CLIENT_IP% \r\n"
                + "echo SERVER_IP is: %SERVER_IP% \r\n"
                + "echo GATEWAY_IP is: %GATEWAY_IP% \r\n"
                + "echo SUBNETMASK is: %SUBNETMASK% \r\n"
                + "echo BOOTMAC is: %BOOTMAC% \r\n"
                + "echo UUID is: %UUID% \r\n" 
                + "echo Parameter a is: {$GetActivityParameter(\"a\")$} \r\n");

        HashMap paramMap = new HashMap();
        paramMap.put("a", "1234");
        paramMap.put("b", "5678");
        
        //Note that the files2Patch must have the full path from / 
        String[] files2Patch = new String[] { "/TEMP/file0.txt", "/autoexec.bat", 
                "/NET/network.ini", "/NET/protocol.ini" };
        
        RBPrebootExecuteTemplate prebootExTempl;
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        if (args.length == 4) {
            String floppyImageName = args[3];
            prebootExTempl = rbServer.registerPrebootExecuteTemplate("preboot Desc",
                    RBPrebootExecuteTemplate.OSTYPE_DOS, RBPrebootExecuteTemplate.FSTYPE_FAT12, 
                    floppyImageName, false, ramDiskSize, files2Add, files2Patch, paramMap);
            System.out.println("\nCreated new PrebootExecuteTemplate for floppy image: " 
                    + floppyImageName + ", description: " + prebootExTempl.getDescription());
        } else {
            System.out.println("\nRetrieving list of available floppy images...");
            RBSoftwarePackageRecs recs = rbServer.getSoftwarePackages(
                    "[Type]='ramd' and [Content]='dos'", "");
            if (recs.size() == 0) {
                System.out.println("No floppy image found on the server, exiting.");
                return;
            }
            RBSoftwarePackage[] floppies = recs.subList(0, recs.size()); 
            for (int i=0; i<floppies.length; i++) {
                RBSoftwarePackage current = floppies[i];
                System.out.println(i + ") " + current.getDescription() 
                        + " pkg: " + current.getSourceFileName());
            }
            int answer = -1;
            do {
                System.out.println("Please enter the number for the image you want to use: \n");
                answer = Integer.parseInt(inReader.readLine().trim());
            } while ( answer < 0 || answer >= recs.size() );
            RBSoftwarePackage floppy = floppies[answer];
            prebootExTempl = rbServer.registerPrebootExecuteTemplate("preboot Desc",
                    floppy, RBPrebootExecuteTemplate.FSTYPE_FAT12, 
                    false, ramDiskSize, files2Add, files2Patch, paramMap);
            System.out.println("\nCreated new PrebootExecuteTemplate for floppy image: " 
                    + floppy.getSourceFileName() + ", description: " + floppy.getDescription());
        }
        
        System.out.println("\nThe PrebootExecuteTemplate will patch the following files on the image: " +
                files2Patch[0] + ", " + files2Patch[1] + ", " + files2Patch[2] + ", " + files2Patch[3]);
        
        RBHostRecs hostRecs = null;
        while (true) {
            System.out.println("\nRetrieving list of available hosts...");
            // Now get list of hosts we can run it on
            hostRecs = rbServer.getHosts("", "");
            
            if (hostRecs.size() == 0) {
                System.out.println("\n\nThere are no hosts found!  This test script needs at least one host to run properly.");
                System.out.println("Please start a host so it boots off the TPMfOSd server and gets registered in the database." +
                        "\nPress enter to continue.");
                inReader.readLine();
            } else {
                break;
            }
        }
        
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress()
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host you wish to preboot with the floppy image: \n");
            inReader = new BufferedReader(new InputStreamReader(System.in));
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = hostRecs.elementAt(hostIndex);
        
        System.out.println("\n\nNow we will schedule the previously created " +
                "ActivityTemplate to run on the host with IP: " 
                + rbhost.getIPAddress()
                + " and Description: " + rbhost.getDescription());
        
        
        /* Schedule Activity to use the PrebootExecuteTemplate we just created on user specified host */
        /* NEW - registering this activity with doWakeOnLan=true */
        RBActivity newAct = rbServer.registerActivity("Test Activity using DOS floppy", prebootExTempl, false, true, null, rbhost);
        System.out.println("\nCreated new activity that uses the RBPrebootExecuteTemplate " +
                "we just created to run immediately on host with IP: " 
                + rbhost.getIPAddress());
        printActivity(newAct); 
        
        System.out.println("Finished. Watch DOS preboot activity on PXE client.");
        
        System.out.println("\nPress enter when the DOS preboot is finished and I \n" +
                "will remove the template to clean up.");
        inReader.readLine();
        
        /* Now remove the template */
        /* Note that if you press enter above and allow this template removal
         * to happen before the activity is complete, removing the template 
         * also removes all record of the activity itself - you should only 
         * remove the template if you are sure the activity is done -- in this 
         * case we are simply doing it to clean up templates created in our 
         * example classes.
         */
        System.out.println("Removing the previously created template...");
        rbServer.removeTemplate(prebootExTempl, true);
        System.out.println("Done.");
    }
    
    private static byte[] readFile(String filename) throws FileNotFoundException, IOException {
        // Add a binary file to the files to add
        File binFile = new File(filename);
        long binFileLength = binFile.length();
        if (binFileLength > Integer.MAX_VALUE) {
            System.out.println("bin File is too large!");
            System.exit(1);
        }
        byte[] filebytes = new byte[(int) binFileLength];
        
        DataInputStream in = new DataInputStream(new FileInputStream(binFile));
        int bytesRead = 0;
        int pos = 0;
        while ( pos < binFileLength && (bytesRead = in.read(filebytes, pos, filebytes.length-pos)) >= 0) {
            pos = pos + bytesRead;
        }
        
        return filebytes;
    }
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
    
    public static void printActivity(RBActivity activity) {
        System.out.println("\n### RBActivity - printing out ###");
        System.out.println("###  getDescription(): " + activity.getDescription());
        System.out.println("###  getDoneDate(): " + activity.getDoneDate());
        System.out.println("###  getExecDate(): " + activity.getExecDate());
        System.out.println("###  getExpireDate(): " + activity.getExpireDate());
        System.out.println("###  getProgress(): " + activity.getProgress());
        System.out.println("###  isSuspendOnError(): " + activity.isSuspendOnError());
    }
    
    public static void printActivityTarget(RBActivityTarget target) {
        System.out.println("\n### RBActivityTarget - printing out ###");
        System.out.println("###  getProgress(): " + target.getProgress());
        System.out.println("###  getStatus(): " + target.getStatus());
    }
}
