/* TestUnattendedProfileCreation.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestUnattendedProfileCreation.java#1 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTargetResults;
import com.ibm.rbapi.RBEvent;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBLinuxBaseSoftwareCluster;
import com.ibm.rbapi.RBLinuxSoftwareGroup;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBProfileCreationConfiguration;
import com.ibm.rbapi.RBProfileCreationParams;
import com.ibm.rbapi.RBProfileCreationResults;
import com.ibm.rbapi.RBProfileCreationTemplate;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBWebExtensionStatus;
import com.ibm.rbapi.exception.RBTimeoutException;

/*
 * This class is a basic example of creating an unattended system profile
 * (sometimes referred to as a "scripted install").  
 * 
 * The profile creation activity happens using an RBHost that is running
 * the Web Interface Extension.  When the test program asks for an RBHost
 * to run the activities against, you must pick an RBHost that is running
 * the WIE and connected to the RBServer.
 * 
 * There are 2 basic steps for creating the unattened system profile.
 * 
 * 1) Probe the installation files to get information about the OS to be
 * packaged (installation files would normally be the contents of the installation
 * CD).  This probe will return an RBProfileCreationParams object containing
 * some details about the RBSystemProfile (and corresponding RBConfiguration(s))
 * to be created.
 * 
 * 2) Perform the actual system profile creation.  This may involve multiple
 * steps for Linux profiles for example, if multiple CDs are needed.
 */
public class TestUnattendedProfileCreation {

    public static void main(String[] args) 
    throws UnknownHostException, RBServerException, RBObjectNotFoundException, 
    IOException, InterruptedException, RBTimeoutException
    {
        if (args.length < 3) {
            System.out.println("usage: TestUnattendedProfileCreation <ip> <port> <password>");
            return;
        }
        System.out.println("TestUnattendedProfileCreation starting...");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String token = args[2];
        RBServer rbServer = new RBServer(ip, port, token); 
        System.out.println("Created rbServer.");
        
        rbServer.isReady(15000);
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = null;
        while (true) {
            System.out.println("\nRetrieving list of available hosts...");
            // Now get list of hosts we can run it on
            hostRecs = rbServer.getHosts("", "");
            
            if (hostRecs.size() == 0) {
                System.out.println("\n\nThere are no hosts found!  This test script needs at least one host running the WIE to run properly.");
                System.out.println("Please start a WIE to communicate with the TPMfOSd server and gets registered in the database." +
                "\nPress enter to continue.");
                inReader.readLine();
            } else {
                break;
            }
        }
        
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress() 
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;

        do {
            System.out.println("Please enter the number for the host running the WIE you wish to perform an " +
                    "unattended profile probe on: \n");
            inReader = new BufferedReader(new InputStreamReader(System.in));
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = availHosts[hostIndex];
        
        // Check if Web Extension is ready on that host
        System.out.println("Checking RBServer.isWebExtensionReady() for host: " + rbhost.getIPAddress() + "...");
        RBWebExtensionStatus wieStatus = null;
        for (int j=0; j<10; j++) {  // try at most 10 times (20 seconds)
            wieStatus = rbServer.isWebExtensionReady(rbhost.getIPAddress());
            if (wieStatus.isPresent()) {
                break;
            }
            Thread.sleep(2000);
        }
        System.out.println("WIE - isPresent(): " + wieStatus.isPresent());
        System.out.println("WIE - getWebExtensionVersion(): " + wieStatus.getWebExtensionVersion());
        System.out.println("WIE - getWebExtensionBuild(): " + wieStatus.getWebExtensionBuild());
        System.out.println("WIE - getServerVersion(): " + wieStatus.getServerVersion());
        System.out.println("WIE - getServerBuild(): " + wieStatus.getServerBuild());
        
        if (!wieStatus.isPresent()) {
            System.out.println("The Web Interface Extension is not detected on the target host.  " +
                    "Please ensure that it is started and run this test again.");
            System.exit(1);
        }
        
        String type = "";
        while (!type.equals("1") && !type.equals("2") && !type.equals("3")) {
            System.out.println("\n\n" +
                    "1) windows\n" +
                    "2) vista\n" +
                    "3) linux\n" +
            "Enter the number for the unattended setup profile type do you wish to create: \n");
            type = inReader.readLine().trim();
        }
        
        String upType = "";
        switch (Integer.parseInt(type)) {
            case 1:
                upType = "windows";
                break; 
            case 2:
                upType = "vista";
                break;
            case 3:
                upType = "linux";
                break;
            default:
                System.out.println("Error determining type - exiting.");
                System.exit(1);
                break;
        }
        
        System.out.println("\nEnter the path to where your installation files exist " +
                " on your RBHost running the WIE\n" + 
                "(this can be a CDROM drive for example): \n");
        String installFolder = inReader.readLine().trim();
        
        /* 
         * Step 1 - probe the contents of the install media on the RBHost
         * running the WIE.
         */
        System.out.println("\nWe will now run a unattended profile probe for a " +
                upType + " install on the host..."); 
        RBProfileCreationParams profCreateParams = null;
        boolean successful = false;
        int count = 0;
        int timeout = 60;
        while (count <= 3) {
            try {
                count++; 
                System.out.println("Probing the with timeout of: " + timeout + " seconds."); 
                profCreateParams = rbServer.probeUnattendedProfile(rbhost, upType, installFolder, timeout);
                if(profCreateParams.getStatus().equals("COMPLETED")) {
                    System.out.println("Probe completed successfully.");
                    successful = true;
                } else {
                    System.out.println("Unexpected probe status - Status returned is: " + profCreateParams.getStatus());
                    printMap(profCreateParams.getError());
                }
                break;
            } catch (RBTimeoutException tme) {
                System.out.println("Probe timed out.");
                timeout = timeout + 20;
            }
        }
        
        if (!successful) {
            System.out.println("Probe did NOT complete successfully.");
            System.exit(2);
        }
        
   
        
        System.out.println("\nNow let's try to register the system profile creation...");
        System.out.println("Registering the profile creation template....");
        RBProfileCreationTemplate pcTempl = rbServer.registerProfileCreationTemplate(
                "test prof creation - unattended", profCreateParams, 
                false, false, false, false);
        
        
        String description = "New test unattended profile";
        System.out.println("Will set the description of the system profile we create to: '"+description+"'.");
        profCreateParams.setDescription(description);
        
        /*
         * Let's look at the profile creation params a little bit..
         */
        RBProfileCreationConfiguration[] profCreateConfs = profCreateParams.getConfigurations();
        System.out.println("ProfileCreationParams contains: " + profCreateConfs.length + " configurations.");
        for (int i=0; i<profCreateConfs.length; i++) {
            System.out.println("  ### Config["+i+"]: ###");
            System.out.println("------------------------");
            System.out.println("    Description: " + profCreateConfs[i].getDescription());
            System.out.println("    TimeZone: " + profCreateConfs[i].getTimeZone());
            System.out.println("    Locale: " + profCreateConfs[i].getLocale());
            
            // Let's change some of them...Note that you can also do this
            // later after profile creation is complete by editing your newly 
            // created RBConfiguration
            if (!upType.equals("linux")) {
                System.out.println("    OrgName: " + profCreateConfs[i].getOrgName());
                System.out.println("    FullName: " + profCreateConfs[i].getFullName());
                System.out.println("\n    Enter the name of the registered owner (FullName), or press enter to leave as-is:\n");
                String fullName = inReader.readLine().trim();
                if (!fullName.equals("")) {
                    profCreateConfs[i].setFullName(fullName);
                }
            }
            
            System.out.println("\n    Enter the admin password you want to use, or press enter to leave blank:\n");
            String adminPwd = inReader.readLine().trim();
            profCreateConfs[i].setAdminPassword(adminPwd);
            
            /*
             * For Linux profile creation, we can possibly select the base
             * software cluster to install, and also additional software groups
             * to install.
             */
            if (upType.equals("linux")) {
                System.out.println("    Config["+i+"] Linux base software cluster: " 
                        + profCreateConfs[i].getBaseSoftwareCluster());
                System.out.println("    Config["+i+"] Linux software groups: " 
                        + profCreateConfs[i].getAdditionalSoftwareGroups());
                
                RBLinuxBaseSoftwareCluster[] linBase = profCreateParams.getAvailLinuxBaseSoftwareClusters();
                if (linBase.length > 0) {
                    System.out.println("    Choose a base software cluster from the list: \n");
                    for (int j=0; j<linBase.length; j++) {
                        System.out.println("    "+j+") " + linBase[j].getIdentifier() + ": " 
                                + linBase[j].getDescription());
                    }
                    System.out.println("    "+linBase.length+") Leave as is ("
                            + profCreateConfs[i].getBaseSoftwareCluster() + ")");
                    String baseChoice = inReader.readLine().trim();
                    
                    try {
                        int baseChoiceInt = Integer.parseInt(baseChoice);
                        if (baseChoiceInt >=0 && baseChoiceInt < linBase.length) {
                            System.out.println("    Changing base software cluster to " + linBase[baseChoiceInt].getIdentifier());
                            profCreateConfs[i].setBaseSoftwareCluster(linBase[baseChoiceInt]);
                        } else {
                            System.out.println("    Leaving base software cluster as is.");
                        }
                    } catch (NumberFormatException nfe) {
                        System.out.println("    Bad input, not changing base software cluster.");
                    }
                }
                
                RBLinuxSoftwareGroup[] linGrp = profCreateParams.getAvailLinuxSoftwareGroups();
                if (linGrp.length > 0) {
                    ArrayList grpArray = new ArrayList();
                    addSoftwareGroupsToConf(profCreateConfs[i], linGrp, inReader, grpArray);
                }
                
            } else {
                // Windows install, let's ask for the product key
                System.out.println("    \n    Enter the product key (e.g. AAAA-BBBB-CCCC-DDDD-EEEE):\n");
                String prodKey = inReader.readLine().trim();
                
                profCreateConfs[i].setProductKey(prodKey);
            }
        }
        
        /*
         * Step 2 - actually create the system profile using the (possibly customized)
         * RBProfileCreationParams.  This might involve multiple steps for
         * Linux system profiles.
         */
        
        // Insert our customized profile creation params into a param map
        // to pass into the activity
        HashMap paramMap = new HashMap();
        paramMap.put("RBProfileCreationParams", profCreateParams);
        RBActivity pcAct = null;
        
        boolean profileCreationComplete = false;
        String pcActDesc = "test unattended prof creation activity - step: ";
        int step = 0;
        while (!profileCreationComplete) { 
            step++;
            Date now = new Date();
            System.out.println("Registering profile creation activity to run immediately - step: " + String.valueOf(step));
            pcAct = rbServer.registerActivity(pcActDesc+String.valueOf(step), 
                    pcTempl, true, false, paramMap, rbhost);

            RBActivityTargetResults actTargetResults = checkForActivityFinished(pcAct, now);

            /* We were just doing a profile creation, so let's get the specific profile creation results.
             * We know the activity was successful because it's checked for in checkForActivityFinished()
             * But we still need to check to see if the profile is completely done being created
             * (For linux profiles for example, we may still have to complete the profile with
             * files from additional CDs).
             */
            RBProfileCreationResults profCreateResults = new RBProfileCreationResults(actTargetResults);
            
            // Print out some info about the profCreateResults
            System.out.println("ProfileCreationResults.isProfileCreationCompleted(): " + profCreateResults.isProfileCreationCompleted()); 
            if (upType.equals("linux")) {
                System.out.println("ProfileCreationResults.getFilesFoundCount(): " + profCreateResults.getFilesFoundCount());
                System.out.println("ProfileCreationResults.getFilesMissingCount(): " + profCreateResults.getFilesMissingCount());
                System.out.println("ProfileCreationResults.getSampleMissingFiles(): "); 
                String[] sampleFiles = profCreateResults.getSampleMissingFiles();
                for (int k=0; k<sampleFiles.length; k++) {
                    System.out.println("    " + sampleFiles[k]);
                }
            }
            
            if (profileCreationComplete = profCreateResults.isProfileCreationCompleted()) {
                System.out.println("System profile creation completed.");
                break;
            } else {
                System.out.println("System profile creation is not complete yet.");
                System.out.println("\nPress enter to continue creating the system profile (you will be prompted for the next install media/files)\n " +
                        "OR -- type 'exit' to force completion of the system profile.");
                String answer = inReader.readLine().trim().toLowerCase();
                // We can continue completing the profile, or force the completion of the profile right away.
                if (answer.equals("exit")) {
                    System.out.println("Forcing completion of system profile...Warning - the created system profile will be missing some files.");
                    profCreateParams.setForceCompleted(true);
                } else {
                    System.out.println("Enter the path to your next CD or install files - default will be ["
                            +profCreateParams.getInstallSourceFolder()+"]: \n");
                    String nextFolder = inReader.readLine().trim();
                    if (!nextFolder.equals("")) {
                        System.out.println("Setting the install source folder to: " + nextFolder);
                        profCreateParams.setInstallSourceFolder(nextFolder);
                    }
                    System.out.println("Continuing creation of system profile from source folder: "
                            + profCreateParams.getInstallSourceFolder() + "...");
                }
                
                // Prepare for next step in profile creation
                // We must extract the ID from the profile creation results
                // so that the server knows which profile we are dealing with.
                profCreateParams.setSysID(profCreateResults.getSysID());
                // Just a reminder that the profile creation params need to be in the parameter map for the activity
                paramMap.put("RBProfileCreationParams", profCreateParams);     
            }
        }
        
        if (!profileCreationComplete) {
            System.out.println("\nWe have reached the timeout and have stopped polling to " +
                    "query events.  Please increase the timeout in this program or wait " +
                    "to see if the profile creation completes.\n");
        }
        
        System.out.println("\n\nPress enter and this will cleanup by removing the activity template and exit.");
        inReader.readLine();
        System.out.println("\nRemoving profile creation template...");
        rbServer.removeTemplate(pcTempl, true);
        
        // Before leaving, display RPC statistics            
        System.out.println("Stats for TestUnattendedProfileCreation:");
        System.out.println(rbServer.getStatistics());
        System.out.println("Done.");
    }
    
    
    // polling to see if our profile creation activity completes.  For a better example of how to 
    // monitor activity status, see TestStressSmart.java
    static RBActivityTargetResults checkForActivityFinished(RBActivity act, Date startTime) 
    throws RBServerException, RBObjectNotFoundException, InterruptedException 
    {
        RBActivityTargetResults actTargetResults = null;
        
        /*
         * This section here refreshes the activity target to see if the activity
         * is complete or not.  This is done just to code something quickly, but 
         * is not very efficient as the polling results in making queries to the 
         * server quite often.  For an example of better activity event monitoring,
         * see the TestRBEvents.java example class, or the TestStressSmart.java class.
         */
        /* Now print out some status for the activity running on our Target */
        int count = 0;  // Loop through and exit when we find the first finished event
        Date lastTime = startTime;
        boolean foundCompletedEvent = false;
        while (!foundCompletedEvent && count < 150 ) {
            Thread.sleep(10000);
            System.out.println("Getting all events from server from time: " + lastTime);
            RBEvent[] events = act.getEventsFromServer(lastTime);

            RBEvent e = null;
            for (int i=0; i<events.length; i++) {
                e = events[i];

                System.out.println("Event: Progress=" + e.getProgress() + " Status=" + e.getStatus());

                // Let's get information about events that are completed or failed
                if (e.getStatus().equals(RBActivityTarget.COMPLETED) ||
                        e.getStatus().equals(RBActivityTarget.FAILED)) {
                    System.out.println("Event status is: " + e.getStatus() + " Now let's get some more info from the server\n"); 

                    System.out.println("Getting corresponding RBActivityTarget object");
                    RBActivityTarget rbActTarget = e.getRBActivityTarget();
                    System.out.println("Getting RBActivityTargetResults object");
                    actTargetResults = rbActTarget.getActivityTargetResults();
                    System.out.println("Results: params -- ");
                    printMap(actTargetResults.getParams());
                    System.out.println("Results: errors -- ");
                    printMap(actTargetResults.getError());

                    foundCompletedEvent = true;
                    System.out.println("Exiting loop.");
                }
            }
            if (e != null) {
                lastTime = e.getUpdateTime();
            } else {
                System.out.println("No events found.");
            }
            count++;
        }
        
        if (actTargetResults == null) {
            System.out.println("Did not find results of activity (perhaps we stopped checking too soon?).\n" +
                    "Exiting.");
            System.exit(10);
        }
        if (!actTargetResults.getStatus().equals("COMPLETED")) {
            System.out.println("System profile creation activity did not complete successfully.\n" +
                    "Exiting.");
            System.exit(11);
        }
        return actTargetResults;
    }
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
    
    
    public static void addSoftwareGroupsToConf(RBProfileCreationConfiguration conf, 
            RBLinuxSoftwareGroup[] availGrps, BufferedReader inReader, ArrayList grpsToAdd) throws IOException {
        int grpChoiceInt = 0;
        while (grpChoiceInt >=0 && grpChoiceInt < availGrps.length) {
            if (conf.getAdditionalSoftwareGroups().equals("")) {
                System.out.println("    Current software groups: (empty)");
            } else {
                System.out.println("    Current software groups: " + conf.getAdditionalSoftwareGroups());
            }
            for (int i=0; i<availGrps.length; i++) {
                if (availGrps[i].getNestedSoftwareGroups().length == 0) {
                    System.out.println("    "+i+") " + availGrps[i].getIdentifier() + ": "
                            + availGrps[i].getDescription());
                } else {
                    System.out.println("    "+i+") " + availGrps[i].getDescription() 
                            + " (Parent Group - choose this to see subgroups)");
                }
            }
            System.out.println("    "+availGrps.length+") Done, continue.");
            String grpChoice = inReader.readLine().trim();
            
            try {
                grpChoiceInt = Integer.parseInt(grpChoice);
                if (grpChoiceInt >=0 && grpChoiceInt < availGrps.length) {
                    
                    RBLinuxSoftwareGroup[] nestedLinGrp = availGrps[grpChoiceInt].getNestedSoftwareGroups();
                    if (nestedLinGrp.length > 0) {
                        System.out.println("\n    Groups contained in subgroup: " + availGrps[grpChoiceInt].getDescription() + ": ");
                        // Recursively call - note that you can also add the whole
                        // parent group and all subgroups will be added automatically.
                        // This is much easier to display in a GUI as an option.
                        addSoftwareGroupsToConf(conf, nestedLinGrp, inReader, grpsToAdd);
                    } else {
                        grpsToAdd.add(availGrps[grpChoiceInt]);

                        RBLinuxSoftwareGroup[] newGrps = new RBLinuxSoftwareGroup[grpsToAdd.size()];
                        for (int j=0; j<grpsToAdd.size(); j++) {
                            RBLinuxSoftwareGroup lsg = (RBLinuxSoftwareGroup) grpsToAdd.get(j);
                            newGrps[j] = lsg;
                        }
                        // Set the additional software groups list - it would 
                        // be better to do this just once at the end - we are doing it
                        // each time so we can see the list change for informational 
                        // purposes.
                        conf.setAdditionalSoftwareGroups(newGrps);
                    }
                }
            } catch (NumberFormatException nfe) {
                System.out.println("    Bad input, continuing.");
            }
        } 
    }
}
