/* TestWinPE.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2006
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestWinPE.java#10 $
 */
package examples;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBPrebootExecuteTemplate;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBSoftwarePackage;
import com.ibm.rbapi.RBSoftwarePackageRecs;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestWinPE {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException, RBTimeoutException
    {
        
        // If the test program is called with 4 arguments consider that args[3] is
        // the selected image filename in files/global/updates (deprecated).
        // If there are only three arguments, get the list of WinPE images from the
        // server and let the user choose one.

        if (args.length < 3 || args.length > 4) {
            System.out.println("usage: TestWinPE <ip> <port> <password> [<winpeimagefile>]\n");
            System.out.println(" Where winpeimagefile is the filename of the WinPE image on "
                    +"your TPMfOSd server in files/global/updates");
            return;
        }

        System.out.println("TestWinPE starting. We will test booting a WinPE image "
                +"on a host machine");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        // Abort if the server is not ready after 15 seconds
        rbServer.isReady(15000);

        RBPrebootExecuteTemplate prebootExTempl;
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        int ramDiskSize = 300*1024; // 300 MB
        HashMap paramMap = new HashMap();
        if (args.length ==4) {
            String imageName = args[3];
            prebootExTempl = rbServer.registerPrebootExecuteTemplate("preboot Desc",
                    RBPrebootExecuteTemplate.OSTYPE_WINPE, RBPrebootExecuteTemplate.FSTYPE_NTFS,
                    imageName, false, ramDiskSize,null,null,paramMap);
            System.out.println("\nCreated new template for WinPE image: " + imageName
                    + ", description: " + prebootExTempl.getDescription());
        } else {
            System.out.println("\nRetrieving list of available WinPE images...");
            RBSoftwarePackageRecs recs = rbServer.getSoftwarePackages(
                    "[Type]='ramd' and [Content]='winpe'", "");
            if (recs.size() == 0) {
                System.out.println("No WinPE image found on the server, exiting.");
                return;
            }
            RBSoftwarePackage[] images = recs.subList(0, recs.size()); 
            for (int i=0; i<images.length; i++) {
                RBSoftwarePackage current = images[i];
                System.out.println(i + ") " + current.getDescription() + " pkg: " 
                        + current.getSourceFileName());
            }
            int answer = -1;
            do {
                System.out.println("Please enter the number for the image you want to use: \n");
                answer = Integer.parseInt(inReader.readLine().trim());
            } while ( answer < 0 || answer >= recs.size() );
            RBSoftwarePackage selected = images[answer];
            prebootExTempl = rbServer.registerPrebootExecuteTemplate("preboot Desc",
                    selected, RBPrebootExecuteTemplate.FSTYPE_NTFS, 
                    false, ramDiskSize, null, null, paramMap);
            System.out.println("\nCreated new PrebootExecuteTemplate for selected image: " 
                    + selected.getSourceFileName()+", description: "+selected.getDescription());
        }
        
        RBHostRecs hostRecs = null;
        while (true) {
            System.out.println("\nRetrieving list of available hosts...");
            // Now get list of hosts we can run it on
            hostRecs = rbServer.getHosts("", "");
            
            if (hostRecs.size() == 0) {
                System.out.println("\n\nThere are no hosts found!  This test script needs "
                        + "at least one host to run properly.");
                System.out.println("Please start a host so it boots off the TPMfOSd server "
                        + "and gets registered in the database." 
                        + "\nPress enter to continue.");
                inReader.readLine();
            } else {
                break;
            }
        }
        
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress()
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host you wish to preboot "
                    + "with the WinPE image: \n");
            inReader = new BufferedReader(new InputStreamReader(System.in));
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = hostRecs.elementAt(hostIndex);
        
        System.out.println("\n\nNow we will schedule the previously created " +
                "ActivityTemplate to run on the host with IP: " 
                + rbhost.getIPAddress()
                + " and Description: " + rbhost.getDescription());
        
        /* Schedule Activity to use the PrebootExecuteTemplate we just created */
        RBActivity newAct = rbServer.registerActivity("Test Activity using WinPE image",
                prebootExTempl, false, false, null, rbhost);
        System.out.println("\nCreated new activity that uses the RBPrebootExecuteTemplate we just "
                + "created to run immediately on host with IP: " + rbhost.getIPAddress());
        printActivity(newAct); 
        
        System.out.println("Finished. Watch WinPE preboot activity on PXE client.");
        
        System.out.println("\nPress enter when the WinPE preboot is finished and I \n" +
                "will remove the template to clean up.");
        inReader.readLine();
        
        /* Now remove the template */
        System.out.println("Removing the previously created template...");
        rbServer.removeTemplate(prebootExTempl, true);
        System.out.println("Done.");
    }
    
    private static byte[] readFile(String filename) throws FileNotFoundException, IOException {
        // Add a binary file to the files to add
        File binFile = new File(filename);
        long binFileLength = binFile.length();
        if (binFileLength > Integer.MAX_VALUE) {
            System.out.println("bin File is too large!");
            System.exit(1);
        }
        byte[] filebytes = new byte[(int) binFileLength];
        
        DataInputStream in = new DataInputStream(new FileInputStream(binFile));
        int bytesRead = 0;
        int pos = 0;
        while ( pos < binFileLength 
                && (bytesRead = in.read(filebytes, pos, filebytes.length-pos)) >= 0) 
        {
            pos = pos + bytesRead;
        }
        
        return filebytes;
    }
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
    
    public static void printActivity(RBActivity activity) {
        System.out.println("\n### RBActivity - printing out ###");
        System.out.println("###  getDescription(): " + activity.getDescription());
        System.out.println("###  getDoneDate(): " + activity.getDoneDate());
        System.out.println("###  getExecDate(): " + activity.getExecDate());
        System.out.println("###  getExpireDate(): " + activity.getExpireDate());
        System.out.println("###  getProgress(): " + activity.getProgress());
        System.out.println("###  isSuspendOnError(): " + activity.isSuspendOnError());
    }
    
    public static void printActivityTarget(RBActivityTarget target) {
        System.out.println("\n### RBActivityTarget - printing out ###");
        System.out.println("###  getProgress(): " + target.getProgress());
        System.out.println("###  getStatus(): " + target.getStatus());
    }
}
